@@ -13,7 +13,7 @@ with a few cherry-picks from the current implementation
 (<https://github.com/tjguk/active_directory/blob/master/active_directory.py>)
 
 Rewrite for Python3 with minimized dependencies
-by Rainer Schwarzbach, 2021-02-01
+by Rainer Schwarzbach, 2021-02-05
 
 License: MIT
 
@@ -86,7 +86,7 @@ def signed_to_unsigned(number):
 
 class Convert:
 
-    """A collection of converter methods for LdapEntry properties"""
+    """A collection of converter methods for LdapEntry attributes"""
 
     base_time = datetime.datetime(1601, 1, 1)
     time_never_high_part = 0x7fffffff
@@ -123,7 +123,7 @@ class Convert:
 
     @classmethod
     def to_guid(cls, item):
-        """Return a GUID from an LDAP entry's property"""
+        """Return a GUID from an LDAP entry's attribute"""
         if item is None:
             return None
         #
@@ -202,7 +202,7 @@ class UnsignedIntegerMapping:
         return '<%s: %s>' % (
             self.__class__.__name__,
             ', '.join(
-                '%s <=> %s' % (name, number)
+                '%s \u2194 %s' % (name, number)
                 for (name, number) in self.items()))
 
 
@@ -357,7 +357,8 @@ class RecordSet:
         suitable for output
         """
         return '{\n%s\n}' % (
-            ', '.join('  %s=%r' % field for field in self.dump_fields()))
+            ',\n'.join(
+                '  %s \u21d2 %r' % field for field in self.dump_fields()))
 
 
 class PathComponent:
@@ -568,13 +569,17 @@ SEARCH_FILTERS = {
 
 class LdapEntry:
 
-    """Store a subset of an LDAP entry's properties.
+    """Store a subset of an LDAP entry's attributes.
     Should be instantiated via the produce_entry()
     factory function.
     """
 
-    additional_properties = {'ADsPath'}
-    ignore_properties = {'nTSecurityDescriptor'}
+    additional_attributes = {'ADsPath'}
+    ignored_attributes = {
+        'dSCorePropagationData',
+        'nTSecurityDescriptor',
+        'userParameters'}
+    ignored_types = (memoryview, win32com.client.CDispatch)
     conversions = dict(
         accountExpires=Convert.to_datetime,
         ADsPath=LdapPath.from_string,
@@ -606,66 +611,62 @@ class LdapEntry:
         uSNCreated=Convert.to_datetime)
 
     def __init__(self, com_object):
-        """Store properties form the provided COM object.
-        The property names are determined from the schema,
-        plus the required (cls.)additional_properties,
-        minus the preformance-degrading (cls.)ignore_properties.
-        Additionally, properties that still are COM objects itself
+        """Store the largest part of attributes from the provided
+        COM object. Attribute names are determined from the schema,
+        plus the required (cls.)additional_attributes,
+        minus the preformance-degrading or non-interesting
+        (cls.)ignored_attributes.
+        Additionally, attributes that still are COM objects itself
         or memoryviews after the conversion will be ignored.
         """
         schema = win32com.client.GetObject(com_object.Schema)
-        property_names = tuple(
-            single_property for single_property in
+        attribute_names = (
             set(schema.MandatoryProperties)
             | set(schema.OptionalProperties)
-            | self.additional_properties
-            if single_property not in self.ignore_properties)
+            | self.additional_attributes) - self.ignored_attributes
         self.__case_translation = dict()
-        self.__property_cache = dict()
-        self.__empty_properties = set()
-        for name in property_names:
+        self.__stored_attributes = dict()
+        self.__empty_attributes = set()
+        for name in attribute_names:
             try:
-                com_property = getattr(com_object, name)
+                com_attribute = getattr(com_object, name)
             except AttributeError:
-                logging.warning('Property %r not found', name)
+                logging.error('Attribute %r not found', name)
                 continue
             #
-            self.__case_translation[name.lower()] = name
             try:
-                com_property = self.conversions[name](com_property)
+                com_attribute = self.conversions[name](com_attribute)
             except KeyError:
                 pass
             #
-            if com_property is None:
-                self.__empty_properties.add(name)
+            try:
+                self.__add_value(name, com_attribute)
+            except ValueError:
                 continue
             #
-            self.__add_property(name, com_property)
         #
-        self.items = self.__property_cache.items
+        self.empty_attributes = frozenset(self.__empty_attributes)
+        self.stored_attributes_items = self.__stored_attributes.items
+        self.ldap_url = self.ADsPath.url
 
-    @property
-    def empty_properties(self):
-        """Return a sorted list of empty properties' names"""
-        return sorted(self.__empty_properties)
-
-    @property
-    def ldap_url(self):
-        """Return the LDAP URL of the entry"""
-        return self.ADsPath.url
-
-    def __add_property(self, name, value):
-        """Add a property value only if it is not a
-        COM Object or a memoryview (or a collection of those)
-        """
+    def __add_value(self, name, value):
+        """Add the value or raise a ValueError for ignored types"""
         if isinstance(value, (list, tuple)):
-            if value and isinstance(
-                    value[0], (memoryview, win32com.client.CDispatch)):
-                return
+            try:
+                if isinstance(value[0], self.ignored_types):
+                    raise ValueError
+                #
+            except IndexError:
+                pass
             #
+        elif isinstance(value, self.ignored_types):
+            raise ValueError
         #
-        if not isinstance(value, (memoryview, win32com.client.CDispatch)):
-            self.__property_cache[name] = value
+        self.__case_translation[name.lower()] = name
+        if value is None:
+            self.__empty_attributes.add(name)
+        else:
+            self.__stored_attributes[name] = value
         #
 
     def child(self, single_path_cmponent):
@@ -679,12 +680,12 @@ class LdapEntry:
                      *self.self.ADsPath.components))
 
     def print_dump(self):
-        """Print all non-empty properties in
+        """Print all non-empty attributes in
         (case-sensitive) alphabetical order
         """
         print('%r\n{' % self)
-        for (name, value) in sorted(self.items()):
-            print('  %s \u2192 %r' % (name, value))
+        for (name, value) in sorted(self.stored_attributes_items()):
+            print('  %s \u21d2 %r' % (name, value))
         #
         print('}')
 
@@ -693,9 +694,7 @@ class LdapEntry:
         return self.objectGUID == other.objectGUID
 
     def __getattr__(self, name):
-        """Instance attribute access to the com object's properties
-        via item access
-        """
+        """LDAP entry attribute access via item access"""
         try:
             return self[name]
         except KeyError as error:
@@ -705,14 +704,14 @@ class LdapEntry:
         #
 
     def __getitem__(self, name):
-        """Access the properties as dict members,
+        """Access the attributes as dict members,
         using case-insensitive names
         """
         translated_name = self.__case_translation[name.lower()]
         try:
-            return self.__property_cache[translated_name]
+            return self.__stored_attributes[translated_name]
         except KeyError:
-            if translated_name in self.__empty_properties:
+            if translated_name in self.empty_attributes:
                 return None
             #
         #
@@ -726,7 +725,7 @@ class LdapEntry:
         """Return a representation with the class name
         and the path
         """
-        return "<%s: %s>" % (self.__class__.__name__, str(self.ADsPath))
+        return '<%s: %s>' % (self.__class__.__name__, str(self))
 
     def __str__(self):
         """Return the path"""
@@ -838,7 +837,7 @@ def search(*args,
 
     If 'active' is set to True or False explicitly,
     yield the path only if the userAccountControl
-    property value matches the desired state.
+    attribute value matches the desired state.
 
     if 'search_filter' is not set, determine a search filter
     automatically.
@@ -878,7 +877,7 @@ def search(*args,
                 yield result.ADsPath
             #
         except TypeError:
-            # no userAccountControl property
+            # no userAccountControl attribute
             yield result.ADsPath
         #
     #
@@ -915,7 +914,7 @@ def get_first_entry(*args, **kwargs):
 
 
 def get_first_user(*args, **kwargs):
-    """Find a user by name or other properties
+    """Find a user by name or other attributes
     from the cached root entry
     """
     for found_path in search_users(*args, **kwargs):
